angular.module('beamng.apps')
.directive('colorCycler', ['bngApi',
  function (bngApi) {
  return {
    restrict: 'E',
    template: '<div style="height:100%; width:100%;" class="bngApp" layout="column" layout-align="center center"><div>Change the Speed at which the color is cycled through:</div><md-slider style="width:90%; max-width: 500px;" ng-model="timeBetweenChange" min="1" max="500"></md-slider></div>',
    replace: true,
    link: function (scope, element, attrs) {

      /**
       * Converts an RGB color value to HSL. Conversion formula
       * adapted from http://en.wikipedia.org/wiki/HSL_color_space.
       * Assumes r, g, and b are contained in the set [0, 1] and
       * returns h, s, and l in the set [0, 1].
       *
       * @param   Number  r       The red color value
       * @param   Number  g       The green color value
       * @param   Number  b       The blue color value
       * @return  Array           The HSL representation
       **/
      function toHsl(rgb) {
        var r = rgb[0], g = rgb[1], b = rgb[2];
        var max = Math.max(r, g, b), min = Math.min(r, g, b);
        var h, s, l = (max + min) / 2;

        if (max == min) {
          h = s = 0; // achromatic
        } else {
          var d = max - min;
          s = l > 0.5 ? d / (2 - max - min) : d / (max + min);
          switch (max){
            case r: h = (g - b) / d + (g < b ? 6 : 0); break;
            case g: h = (b - r) / d + 2; break;
            case b: h = (r - g) / d + 4; break;
          }
          h /= 6;
        }

        return [h, s, l];
      }

      /**
       * Converts an HSL color value to RGB. Conversion formula
       * adapted from http://en.wikipedia.org/wiki/HSL_color_space.
       * Assumes h, s, and l are contained in the set [0, 1] and
       * returns r, g, and b in the set [0, 255].
       *
       * @param   Number  h       The hue
       * @param   Number  s       The saturation
       * @param   Number  l       The lightness
       * @return  Array           The RGB representation
       **/
      function fromHsl(hsl) {
        var r, g, b;

        if (hsl[1] === 0) {
          r = g = b = hsl[2]; // achromatic
        } else {
          var hue2rgb = function hue2rgb(p, q, t) {
            if (t < 0) {t += 1;}
            if (t > 1) {t -= 1;}
            if (t < 1 / 6) {return p + (q - p) * 6 * t;}
            if (t < 1 / 2) {return q;}
            if (t < 2 / 3) {return p + (q - p) * (2 / 3 - t) * 6;}
            return p;
          };

          var q = hsl[2] < 0.5 ? hsl[2] * (1 + hsl[1]) : hsl[2] + hsl[1] - hsl[2] * hsl[1];
          var p = 2 * hsl[2] - q;
          r = hue2rgb(p, q, hsl[0] + 1 / 3);
          g = hue2rgb(p, q, hsl[0]);
          b = hue2rgb(p, q, hsl[0] - 1 / 3);
        }
        return [r, g, b];
      }

      scope.$watch('timeBetweenChange', () => {startInterval(scope.timeBetweenChange);});

      var timerInterval
        , currentColor = [0, 1, 0.5];
      scope.timeBetweenChange = 500 - 200;

      function startInterval (time) {
        clearInterval(timerInterval);

        timerInterval = setInterval(() => {
          currentColor[0] = (currentColor[0] + 1/240) % 1;
          if (currentColor[1] < 1) {
            currentColor += 0.01;
          }

          if (currentColor[2] < 0.5) {
            currentColor += 0.01;
          } else if (currentColor[2] > 0.5) {
            currentColor -= 0.01;
          }

          bngApi.engineScript('changeVehicleColor("' + fromHsl(currentColor).join(' ') + '");');
        }, 500 - time);
      }

      function getVehicleColor () {
        bngApi.engineScript('getVehicleColor();', (res) => {
          if (res) {
            var helper = res.split(' ');
            if (helper.length > 1) {
              currentColor = toHsl(helper.map(Number)).slice(0, 3);
            }
          }
        });
      }

      scope.$on('VehicleChange', () => {getVehicleColor();});
      

      // Do stuff when the app closes.
      scope.$on('$destroy', function () {
        if (timerInterval) {
          clearInterval(timerInterval);
        }
      });


      getVehicleColor();
    }
  };
}]);
